(function() {
    // Local Storage TODOs:
    // - Consistent order operation with the server
    // - Compressing requests

    const backingStore = new Map();
    let keys = [];

    function getKey(key) {
        console.log("[LocalStorageTracker] [G]", key);

        if (backingStore.has(key.toString())) {
            return backingStore.get(key.toString());
        }

        return null;
    }

    function setKey(key, value) {
        console.log("[LocalStorageTracker] [S]", key, value);

        fetch("/cgi-bin/rpc/ls/upsert", {
            body: JSON.stringify({
                key: key.toString(),
                value: value.toString()
            }),
            headers: {
                "Content-Type": "application/json"
            },
            method: "POST"
        }).then(res => res.json()).then(r => {
            if (r.ok) {
                console.log("Sync ok");
            } else {
                console.error("Sync fucked");
            }
        })

        if (keys.indexOf(key.toString()) === -1) {
            keys.push(key.toString());
        }

        backingStore.set(key.toString(), value.toString());
    }

    function removeKey(key) {
        console.log("[LocalStorageTracker] [R]", key);

        fetch("/cgi-bin/rpc/ls/remove", {
            body: JSON.stringify({
                key: key.toString()
            }),
            headers: {
                "Content-Type": "application/json"
            },
            method: "POST"
        }).then(res => res.json()).then(r => {
            if (r.ok) {
                console.log("Sync ok");
            } else {
                console.error("Sync fucked");
            }
        })

        if (keys.indexOf(key.toString()) !== -1) {
            keys = keys.filter(k => k !== key.toString());
        }

        backingStore.delete(key);
    }

    function clear() {
        console.log("[LocalStorageTracker] [C]");

        fetch("/cgi-bin/rpc/ls/clear", {
            body: JSON.stringify({}),
            headers: {
                "Content-Type": "application/json"
            },
            method: "POST"
        }).then(res => res.json()).then(r => {
            if (r.ok) {
                console.log("Sync ok");
            } else {
                console.error("Sync fucked");
            }
        })

        keys = [];

        backingStore.clear();
    }

    function getKeyByIndex(index) {
        console.log("[LocalStorageTracker] [K]", index);

        let realIndex = Number(index);
        if (Number.isNaN(realIndex)) {
            return null;
        }

        if (realIndex < 0 || realIndex >= keys.length) {
            return null;
        }

        return keys[realIndex];
    }

    const api = {
        "clear": clear,
        "getItem": getKey,
        "setItem": setKey,
        "removeItem": removeKey,
        "key": getKeyByIndex,
        "__rdh_debug": function() {
            console.log(keys, backingStore);
        }
    }

    Object.defineProperty(window, "localStorage", {
        value: new Proxy({}, {
            set(_, key, value, __) {
                setKey(key, value);
            },
            get(_, k, __) {
                if (api[k]) {
                    return api[k];
                }
    
                if (k === "length") {
                    return keys.length;
                }
    
                return getKey(k);
            },
            ownKeys(_) {
                return [...keys];
            }
        }),
        writable: false,
        configurable: false
    });

    // Populate it with our initial values
    const xhr = new XMLHttpRequest();
    xhr.open("GET", "/cgi-bin/rpc/ls/read_all", false);
    xhr.send(null);

    if (xhr.status !== 200) {
        alert("it appears to be fucked");
        window.stop();
    }

    const responseBody = JSON.parse(xhr.responseText);
    for (let { key, value } of responseBody) {
        keys.push(key);
        backingStore.set(key, value);
    }

    // DIE. DIE. DIE. (IndexedDB is hell to implement. I will do it later, for now zap it away so games fall back to localstorage.)
    Object.defineProperty(window, "indexedDB", {
        value: undefined,
        writable: false,
        configurable: false,
        enumerable: false
    });
})();