import express, { urlencoded } from "express";
import { join } from "path";
import z from "zod";
import { authenticate, sessionValidate } from "./auth.js";
import { getAllGames } from "./game.js";
import { RootDomain } from "./env.js";

const router = express.Router();
const preauthRouter = express.Router();
const postauthRouter = express.Router();

const base = import.meta.dirname;

const loginPayload = z.object({
    username: z.string(),
    password: z.string()
});

preauthRouter.use(urlencoded());
preauthRouter.get("/", (req, res) => {
    res.sendFile(join(base, "statics", "login.html"));
});

preauthRouter.post("/", async (req, res) => {
    const body = loginPayload.parse(req.body);
    const result = await authenticate(body.username, body.password);
    
    if (result === null) {
        res.status(403).send("Bad username or password");
        return;
    }

    res.cookie("rdh_sid", result, {
        httpOnly: true,
        path: "/",
        maxAge: 86400 * 400 * 1000
    });

    res.set('Location', '/');
    res.status(303).send("Welcome");
});

postauthRouter.get("/", (req, res) => {
    let html = `<h1>Available games</h1><ul>`;
    for (let game of getAllGames()) {
        html = html + `<li><a href="${req.protocol}://${game.slug}.${RootDomain}/cgi-bin/rpc/dollhouse_authenticate?sid=${req.cookies["rdh_sid"]}">${game.name}</a></li>`
    }
    html = html + "</ul>"
    res.set("Content-Type", "text/html;charset=UTF-8");
    res.send(html);
});


router.use((req, res, next) => {
    const session = req.cookies["rdh_sid"];
    if (!session) {
        return preauthRouter(req, res, next);
    }

    const username = sessionValidate(session);
    if (username === null) {
        return preauthRouter(req, res, next);
    }
    
    req.active_user = username;
    return postauthRouter(req, res, next);
});

export default router;